@@ -1,4 +1,5 @@
 use std::any::TypeId;
+use std::io;
 
 mod format_parser;
 
@@ -15,9 +16,14 @@ pub struct InputFormat<'a> {
 }
 
 impl<'a> InputFormat<'a> {
-    pub fn new(input: &'a str) -> Self {
-        let (_, elements) = format_parser::tokenize(input).unwrap();
-        return Self { elements };
+    pub fn new(input: &'a str) -> io::Result<Self> {
+        let (_, elements) = format_parser::tokenize(input).map_err(|e| {
+            io::Error::new(
+                io::ErrorKind::InvalidInput,
+                format!("Invalid input format string: {}", e),
+            )
+        })?;
+        return Ok(Self { elements });
     }
 
     pub fn input_strings(&self, input: &'a str) -> Vec<&'a str> {
@@ -26,7 +32,13 @@ impl<'a> InputFormat<'a> {
         let mut input_elements = Vec::new();
         for element in &self.elements {
             match *element {
-                InputFormatToken::Type(_) | InputFormatToken::GenericType => if capture { panic!("oooo") } else { capture = true },
+                InputFormatToken::Type(_) | InputFormatToken::GenericType => {
+                    if capture {
+                        panic!("oooo")
+                    } else {
+                        capture = true
+                    }
+                }
                 InputFormatToken::Text(text) => {
                     if capture {
                         capture = false;
@@ -50,3 +62,35 @@ impl<'a> InputFormat<'a> {
         return input_elements;
     }
 }
+
+#[cfg(test)]
+mod test {
+    use super::*;
+
+    #[test]
+    fn test_simple_generic_formatter() {
+        let formatter = InputFormat::new("{} -> {}").unwrap();
+        assert_eq!(
+            formatter.elements,
+            vec![
+                InputFormatToken::GenericType,
+                InputFormatToken::Text(" -> "),
+                InputFormatToken::GenericType
+            ]
+        )
+    }
+
+    #[test]
+    #[should_panic]
+    fn test_wrong_formatter_unescaped_open_bracket() {
+        let formatter = InputFormat::new("{} -{> {}").unwrap();
+        println!("{:?}", formatter);
+    }
+
+    #[test]
+    #[should_panic]
+    fn test_wrong_formatter_unescaped_close_bracket() {
+        let formatter = InputFormat::new("{} -}> {}").unwrap();
+        println!("{:?}", formatter);
+    }
+}
